require 'common_password/list'
module CommonPassword
  # The CommonPassword module adds a new validation for Devise Models.
  # No modifications to routes or controllers needed.
  # Simply add :uncommon_password to the list of included modules in your
  # devise module, and all new registrations will be blocked if they use
  # a common password.
  module Model
    extend ActiveSupport::Concern
    included do
      validate :not_common_password, if: :should_validate?
    end

    module ClassMethods
      ::Devise::Models.config(self, :password_matches)
    end

    def password_common?
      @password_common_index = nil

      password_common_top.present?
    end

    private

      def should_validate?
        password.present? && password_required? && self.class.common_active && !self.errors.include?(:password)
      end

      def password_common_index
        @password_common_index ||= CommonPassword::List.get.index(password.downcase)
      end

      def password_common_top
        return nil if password_common_index.nil?

        [100, 1_000, 10_000].each do |index|
          return index if password_common_index < index
        end
      end

      def not_common_password
        if password_common?
          errors.add(:password, :"common.top#{password_common_top}")
        end
      end
  end
end
