# frozen_string_literal: true

require 'devise'
require 'zxcvbn_password/devise'
require 'common_password/devise'
require 'pwned_password/devise'
require 'devise/password_breach_alert/model'
require 'password_breach_alert/breaches_policies'
require 'password_breach_alert/breaches_filters'

module Devise
  extend ZxcvbnPassword::Devise
  extend CommonPassword::Devise
  extend PwnedPassword::Devise
  mattr_accessor :pwned_active, :common_active, :zxcvbn_active
  @@pwned_active = true
  @@common_active = true
  @@zxcvbn_active = true

  mattr_accessor :breaches_policy, :breaches_filter, :breaches_filter_options
  @@breaches_policy = 'PasswordBreachAlert::BreachesPolicies::SendDeviseNotification'
  @@breaches_filter = 'PasswordBreachAlert::BreachesFilters::NewWithUser'
  @@breaches_filter_options = -> { {} }

  module PasswordBreachAlert
  end
end

# Load default I18n
#
I18n.load_path.unshift File.join(File.dirname(__FILE__), 'password_breach_alert', 'locales', 'en.yml')
I18n.load_path.unshift File.join(File.dirname(__FILE__), 'password_breach_alert', 'locales', 'it.yml')
