require 'password_breach_alert/api/breach'
require 'password_breach_alert/api/breachedaccount'

module PasswordBreachAlert
  class Checker
    attr_reader :breaches_filter, :breaches_policy

    def self.default_breaches_filter
      options = Devise.breaches_filter_options.call.reverse_merge(api: PasswordBreachAlert::Api::Breachedaccount.new)
      Devise.breaches_filter.constantize.new(options)
    end

    def self.default_breaches_policy
      Devise.breaches_policy.constantize.new
    end

    def initialize(breaches_filter: self.class.default_breaches_filter, breaches_policy: self.class.default_breaches_policy)
      @breaches_filter = breaches_filter
      @breaches_policy = breaches_policy
    end

    def call(users)
      new_breaches = fetch_and_create_new_breaches
      breaches = all_breaches

      users.each do |user|
        user_breaches = breaches_filter.call(user, new_breaches, breaches)

        breaches_policy.call(user, user_breaches)
      end
    end

    private

      def fetch_and_create_new_breaches
        api_breaches = PasswordBreachAlert::Api::Breach.new.call

        PasswordBreachAlert::Models::Breach.create_new_from_api(api_breaches)
      end

      def all_breaches
        PasswordBreachAlert::Models::Breach.all
      end
  end
end
