# == Schema Information
#
# Table name: breaches
#
#  id           :integer          not null, primary key
#  name         :string
#  title        :string
#  domain       :string
#  breach_date  :datetime
#  added_date   :datetime
#  pwn_count    :integer
#  description  :string
#  logo_path    :string
#  data_classes :text
#  created_at   :datetime         not null
#  updated_at   :datetime         not null
#

module PasswordBreachAlert
  # rails generate password_breach_alert
  module Models
    class Breach < ActiveRecord::Base # rubocop:disable ApplicationRecord
      serialize :data_classes, Array

      validates :name, presence: true

      scope :sorted, -> { order('added_date DESC') }

      def self.create_new_from_api(api_breaches)
        created_ids = []
        present_names = Breach.pluck(:name)

        api_breaches.each do |api_breach|
          next if api_breach['Name'].in?(present_names)

          breach = Breach.create(
            name: api_breach['Name'],
            title: api_breach['Title'],
            domain: api_breach['Domain'],
            breach_date: api_breach['BreachDate'],
            added_date: api_breach['AddedDate'],
            pwn_count: api_breach['PwnCount'],
            description: api_breach['Description'],
            logo_path: api_breach['LogoPath'],
            data_classes: api_breach['DataClasses']
          )

          created_ids << breach.id
        end

        Breach.where(id: created_ids).sorted
      end
    end
  end
end
