require "zxcvbn_password/email_tokeniser"
require "zxcvbn_password/errors"
require "ostruct"

module ZxcvbnPassword
  module Model
    extend ActiveSupport::Concern

    # delegate :zxcvbn_min_password_score, to: "self.class"
    # delegate :zxcvbn_tester, to: "self.class"

    included do
      validate :not_zxcvbn_password, if: proc { password_required? && self.class.zxcvbn_active && !self.errors.include?(:password) }
    end

    def zxcvbn_password_score
      @zxcvbn_password_score = self.class.zxcvbn_password_score(self)
    end

    def zxcvbn_password_crack_time
      zxcvbn_password_score.crack_times_display["offline_slow_hashing_1e4_per_second"]
    end

    def password_zxcvbn?
      zxcvbn_password_score.score < ::Devise.zxcvbn_min_password_score
    end

    private

    def not_zxcvbn_password
      if password_zxcvbn?
        errors.add :password, :zxcvbn, i18n_variables
      end
    end

    def i18n_variables
      {
        # feedback: zxcvbn_feedback,
        crack_time_display: zxcvbn_password_crack_time,
        score: zxcvbn_password_score.score,
        # zxcvbn_min_password_score: ::Devise.zxcvbn_min_password_score
      }
    end

    def zxcvbn_feedback
      feedback = zxcvbn_password_score.feedback.values.flatten.reject(&:empty?)
      return "Add another word or two. Uncommon words are better." if feedback.empty?

      feedback.join(". ").gsub(/\.\s*\./, ".")
    end

    class_methods do
      ::Devise::Models.config(self, :zxcvbn_min_password_score)
      ::Devise::Models.config(self, :zxcvbn_tester)

      def zxcvbn_password_score(user, arg_email = nil)
        return raise ZxcvbnPasswordError, "the object must respond to password" unless user.respond_to?(:password)

        password = user.password.to_s

        zxcvbn_weak_words = []

        if arg_email
          zxcvbn_weak_words += [arg_email, *ZxcvbnPassword::EmailTokeniser.split(arg_email)]
        end

        # User method results are saved locally to prevent repeat calls that might be expensive
        if user.respond_to?(:email)
          local_email = user.email
          zxcvbn_weak_words += [local_email, *ZxcvbnPassword::EmailTokeniser.split(local_email)]
        end

        if user.respond_to?(:weak_words)
          return raise ZxcvbnPasswordError, "weak_words must return an Array" unless user.weak_words.is_a?(Array)

          local_weak_words = user.weak_words
          zxcvbn_weak_words += local_weak_words
        end

        ::Devise.zxcvbn_tester.test(password, zxcvbn_weak_words)
      end
    end
  end
end
